package org.elsys.labs.sampleeditor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

public class ElsysListEditor extends EditorPart {

	private ListViewer fListViewer;
	
	@Override
	public void doSave(IProgressMonitor monitor) {
	}
	
	public ListViewer getListViewer () {
		return fListViewer;
	}

	@Override
	public void doSaveAs() {
	}

	private List<String> fContent;
	private boolean fIsDirty = false;
	
	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		fContent = new ArrayList <String> ();
		fIsDirty = false;
		setSite(site);
		setInput(input);
	}

	@Override
	public boolean isDirty() {
		return fIsDirty;
	}

	public void setDirty () {
		fIsDirty = true;
		firePropertyChange(IEditorPart.PROP_DIRTY);
	}
	
	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public void createPartControl(Composite parent) {
		Composite rootComposite = new Group (parent, SWT.NONE);
		rootComposite.setLayout(new GridLayout (1, false));
		GridData data = new GridData (GridData.FILL);
		rootComposite.setLayoutData(data);
		
		fillBody (rootComposite);
		createActions ();
		createContextMenu ();
	}

	private void fillBody (Composite parent) {
		fListViewer = new ListViewer (parent);
		fListViewer.setLabelProvider(new LabelProvider ());
		fListViewer.setContentProvider(new IStructuredContentProvider () {

			public void dispose() {
			}

			public void inputChanged(Viewer viewer, Object oldInput,
					Object newInput) {
			}

			public Object[] getElements(Object inputElement) {
				if (List.class.isInstance(inputElement)){
					return ((List)inputElement).toArray();
				}
				
				return null;
			}
			
		});
		fListViewer.getList().setLayoutData(new GridData (GridData.FILL_BOTH));
		fListViewer.setInput(fContent);
	}
	
	private Action fAddAction;
	private Action fRemoveAction;
	
	private void createActions (){
		fAddAction = new AddAction (fContent, this);
		fAddAction.setText("Add String");
		fRemoveAction = new RemoveAction (fContent, this);
	}
	
	private void createContextMenu () {
		MenuManager menuManager = new MenuManager ("#PopMenu");
		menuManager.setRemoveAllWhenShown(true);
		menuManager.addMenuListener(new IMenuListener () {

			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu (manager);
			}
			
		});
		org.eclipse.swt.widgets.List list = fListViewer.getList();
		Menu menu = menuManager.createContextMenu(list);
		list.setMenu(menu);
		getSite().registerContextMenu(menuManager, fListViewer);
	}
	
	private void fillContextMenu (IMenuManager manager) {
		manager.add(fAddAction);
		manager.add(fRemoveAction);
	}
	
	@Override
	public void setFocus() {
	}

}
